import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import com.ibm.xml.parsers.DOMParser;
import org.w3c.dom.*;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import ui.DOMTree;

/** SimpleTreeView4 builds on SimpleTreeView3
 *  to show how to highlight errors in the JTree and
 *  associate the error by JTree TreeNode and DOM Node.
 */
public class SimpleTreeView4 extends JFrame {

    DOMParser parser;
    Errors errors;
    JTextArea messageText;
    DOMTree jtree;
    
    /** main */
    public static void main (String args[]) {

        if (args.length > 0) {
            String filename = args[0];
            try {
                SimpleTreeView4 frame = new SimpleTreeView4(filename) ;
                frame.addWindowListener(new java.awt.event.WindowAdapter() {
                 public void windowClosing(java.awt.event.WindowEvent e) {
                     System.exit(0);
                 }
                });
                frame.setSize(600, 400);
                frame.setVisible(true);
                } catch (Exception e) {
                    e.printStackTrace(System.err);
                }
        }
    }

    /** Constructor */
    public SimpleTreeView4 (String filename) {
        super("SimpleTreeView4: "+filename);
        try {
            parser = new DOMParser();
            errors = new Errors();
            parser.setNodeExpansion(DOMParser.FULL);
            parser.setErrorHandler(errors);
            parser.setContinueAfterFatalError(true);
            parser.parse(filename);
            Document document = parser.getDocument();

            // jtree  UI setup
            jtree = new DOMTree(document);
            jtree.setCellRenderer(new XMLTreeCellRenderer());
            jtree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

            // Listen for when the selection changes, call nodeSelected(node)
            jtree.addTreeSelectionListener(
                new TreeSelectionListener() {
                    public void valueChanged(TreeSelectionEvent e) {
                        TreeNode node = (TreeNode)
                            (e.getPath().getLastPathComponent());

                        nodeSelected(node);
                    }
                }
            );
            
            //expandTree();
            JPanel treePanel = new JPanel(new BorderLayout());
            treePanel.add(new JScrollPane(jtree) {
                    public Dimension getPreferredSize(){
                        Dimension size = SimpleTreeView4.this.getSize();
                        return new Dimension(size.width, size.height/2 );
                    }
                }, BorderLayout.CENTER);

            treePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Tree View"),
                BorderFactory.createEmptyBorder(4, 4, 4, 4)
                ));

            // message text UI setup
            messageText = new JTextArea(3,5);

            Enumeration elements = errors.getErrorNodes().elements();
            while (elements.hasMoreElements()) {
                messageText.append( (String)elements.nextElement());
            }
            
            JPanel messagePanel = new JPanel(new BorderLayout());
            messagePanel.add(new JScrollPane(messageText));
            messagePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Messages"),
                BorderFactory.createEmptyBorder(4, 4, 4, 4)
                ));
            
            JComponent split = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                true, treePanel, messagePanel);
            
            getContentPane().add(split);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }
        
    /** called when our JTree's nodes are selected.
     */
    void nodeSelected(TreeNode treeNode) {

        Node node = jtree.getNode(treeNode);
        StringBuffer sb = new StringBuffer();
        
        messageText.selectAll();
        messageText.cut();
        
        Object errorObject = errors == null ? null : errors.getError(node);
        if (errorObject != null) {
            // There *is* an error in this node.
            messageText.setForeground(Color.red);
            sb.append(errorObject);
        } else {
            messageText.setForeground(Color.black);
        }

        if (node.getNodeType() == Node.ELEMENT_NODE) {
            sb.append("Element= ");
            sb.append(node.getNodeName());
            sb.append("\n");
            NamedNodeMap attrs = node.getAttributes();
            if (attrs != null) {
                int length = attrs.getLength();
                for (int i = 0; i < length; i++) {
                    Attr attr = (Attr)attrs.item(i);
                    sb.append("  attribute name=");
                    sb.append(attr.getName());
                    sb.append(" attribute value=");
                    sb.append(attr.getValue());
                    sb.append("\n");
                }
            }
        } else 
        if (node.getNodeType() == Node.TEXT_NODE) {
            sb.append("Text= ");
            sb.append(node.getNodeValue());
        }

        messageText.append(sb.toString());
    }
    
    /** Utility function to expand the jtree */
    void expandTree() {
        for (int i = 0; i < jtree.getRowCount(); i++) {
            jtree.expandRow(i);
        }
    }

    /** Store the errors as a Hashtable with Node keys. */
    class Errors implements ErrorHandler {

        Hashtable errorNodes = new Hashtable();

        public void warning(SAXParseException ex) {
            store(ex, "[Warning]");
        }

        public void error(SAXParseException ex) {
            store(ex, "[Error]");
        }

        public void fatalError(SAXParseException ex) throws SAXException {
            store(ex, "[Fatal Error]");
        }

        public Hashtable getErrorNodes() {
            return errorNodes;
        }

        public Object getError(Node node) {
            return errorNodes.get(node);
        }

        public void resetErrors() {
            errorNodes.clear();
        }

        void store(SAXParseException ex, String type) {

            // build error text
            String errorString= type+" at line number, "+ex.getLineNumber()
                +": "+ex.getMessage()+"\n";

            Node currentNode = parser.getCurrentNode();
            if (currentNode == null) return;

            // accumulate any multiple errors per node in the Hashtable.
            String previous = (String) errorNodes.get(currentNode);
            if (previous != null)
                errorNodes.put(currentNode, previous +errorString);
            else
                errorNodes.put(currentNode, errorString);
        }
    }
    
    /*
     * The XMLTreeCellRenderer is an inner class which enables the
     * highlighting of errors in the tree.
     */
    class XMLTreeCellRenderer extends DefaultTreeCellRenderer
    {

        public Component getTreeCellRendererComponent(JTree tree, Object value,
                          boolean selected, boolean expanded,
                          boolean leaf, int row,
                              boolean hasFocus)
        {
            Node node = ((DOMTree)tree).getNode(value);
            Component comp = super.getTreeCellRendererComponent(tree, value,
                           selected,  expanded, leaf,  row, hasFocus);
            if (errors != null && errors.getErrorNodes() != null && value != null && node != null
             && errors.getErrorNodes().containsKey( node )) {
                comp.setForeground(Color.red);
            }
            return comp;
        }
    }
    
}
